@@ -23,9 +23,9 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py 3560 2008-02-11 17:35:26Z wwalker $"
-__version__   = "$Revision: 3560 $"
-__date__      = "$Date: 2008-02-11 12:35:26 -0500 (Mon, 11 Feb 2008) $"
+__id__        = "$Id: braillegenerator.py 3784 2008-04-01 01:24:54Z richb $"
+__version__   = "$Revision: 3784 $"
+__date__      = "$Date: 2008-03-31 21:24:54 -0400 (Mon, 31 Mar 2008) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -1165,7 +1165,11 @@ class BrailleGenerator:
                     continue
                 else:
                     for j in range(0, action.nActions):
-                        if action.getName(j) == "toggle":
+                        # Translators: this is the action name for
+                        # the 'toggle' action. It must be the same
+                        # string used in the *.po file for gail.
+                        #
+                        if action.getName(j) in ["toggle", _("toggle")]:
                             hasToggle[i] = True
                             break
 
@@ -1201,7 +1205,12 @@ class BrailleGenerator:
                 debug.println(debug.LEVEL_FINEST,
                     "braillegenerator._getBrailleRegionsForTableCell " \
                     + "looking at action %d" % i)
-                if action.getName(i) == "toggle":
+
+                # Translators: this is the action name for
+                # the 'toggle' action. It must be the same
+                # string used in the *.po file for gail.
+                #
+                if action.getName(i) in ["toggle", _("toggle")]:
                     regions = self._getBrailleRegionsForCheckBox(obj)
 
                     # If this table cell doesn't have any label associated
@@ -1320,6 +1329,12 @@ class BrailleGenerator:
                 focusRowRegion = None
                 for i in range(0, table.nColumns):
                     cell = table.getAccessibleAt(row, i)
+                    if not cell:
+                        debug.println(debug.LEVEL_WARNING,
+                             "ERROR: braillegenerator." \
+                             + "_getBrailleRegionsForTableCellRow" \
+                             + " no accessible at (%d, %d)" % (row, i))
+                        continue
                     state = cell.getState()
                     showing = state.contains(pyatspi.STATE_SHOWING)
                     if showing:
