@@ -19,9 +19,9 @@
 
 """Provides the default implementation for flat review for Orca."""
 
-__id__        = "$Id: flat_review.py 3339 2007-12-04 21:35:49Z richb $"
-__version__   = "$Revision: 3339 $"
-__date__      = "$Date: 2007-12-04 16:35:49 -0500 (Tue, 04 Dec 2007) $"
+__id__        = "$Id: flat_review.py 3831 2008-04-15 23:35:20Z eitani $"
+__version__   = "$Revision: 3831 $"
+__date__      = "$Date: 2008-04-15 19:35:20 -0400 (Tue, 15 Apr 2008) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -531,10 +531,19 @@ class Line:
                 # The 'isinstance(zone, TextZone)' test is a sanity check
                 # to handle problems with Java text. See Bug 435553.
                 if isinstance(zone, TextZone) and \
-                   ((zone.accessible.getRole() == pyatspi.ROLE_TEXT) \
-                    or (zone.accessible.getRole() == \
-                                                pyatspi.ROLE_PASSWORD_TEXT) \
-                    or (zone.accessible.getRole() == pyatspi.ROLE_TERMINAL)):
+                   ((zone.accessible.getRole() in \
+                         (pyatspi.ROLE_TEXT,  
+                          pyatspi.ROLE_PASSWORD_TEXT,
+                          pyatspi.ROLE_TERMINAL)) or \
+                    # [[[TODO: Eitan - HACK: 
+                    # This is just to get FF3 cursor key routing support.
+                    # We really should not be determining all this stuff here,
+                    # it should be in the scripts. 
+                    # Same applies to roles above.]]]
+                    (zone.accessible.getRole() in \
+                         (pyatspi.ROLE_PARAGRAPH,
+                          pyatspi.ROLE_HEADING,
+                          pyatspi.ROLE_LINK))):
                     region = braille.ReviewText(zone.accessible,
                                                 zone.string,
                                                 zone.startOffset,
@@ -702,7 +711,7 @@ class Context:
                 zone = line.zones[zoneToCheck]
                 currentLineIndex = lineToCheck
                 currentZoneIndex = zoneToCheck
-                if caretOffset:
+                if caretOffset and zone.words:
                     currentWordIndex = len(zone.words) - 1
                     currentCharIndex = \
                           zone.words[currentWordIndex].length - 1
@@ -871,11 +880,12 @@ class Context:
 
         debug.println(debug.LEVEL_FINEST, "  looking at text:")
 
-        text = accessible.queryText()
-        if text:
-            zones = []
-        else:
+        try:
+            text = accessible.queryText()
+        except NotImplementedError:
             return []
+        else:
+            zones = []
 
         length = text.characterCount
 
@@ -941,6 +951,22 @@ class Context:
             textZones = self.splitTextIntoZones(
                 accessible, string, startOffset, cliprect)
 
+            # We need to account for the fact that newlines at the end of 
+            # text are treated as being on the same line when they in fact
+            # are a whole separate blank line.  So, we check for this and
+            # make up a new text zone for these cases.  See bug 434654.
+            #
+            if (endOffset == length) and (string[-1:] == "\n"):
+                [x, y, width, height] = text.getRangeExtents(startOffset, 
+                                                             endOffset,
+                                                             0)
+                if not textZones:
+                    textZones = []
+                textZones.append(TextZone(accessible,
+                                          endOffset,
+                                          "",
+                                          x, y + height, 0, height))
+
             if textZones:
                 zones.extend(textZones)
             elif len(zones):
@@ -1035,7 +1061,11 @@ class Context:
             if action:
                 hasToggle = False
                 for i in range(0, action.nActions):
-                    if action.getName(i) == "toggle":
+                    # Translators: this is the action name for
+                    # the 'toggle' action. It must be the same
+                    # string used in the *.po file for gail.
+                    #
+                    if action.getName(i) in ["toggle", _("toggle")]:
                         hasToggle = True
                         break
                 if hasToggle:
@@ -1288,7 +1318,11 @@ class Context:
             return []
 
         zones = []
-        rootexts = root.queryComponent().getExtents(0)
+        try:
+            rootexts = root.queryComponent().getExtents(0)
+        except:
+            return []
+
         rootrole = root.getRole()
 
         # If we're at a leaf node, then we've got a good one on our hands.
@@ -1323,6 +1357,15 @@ class Context:
            or (rootrole == pyatspi.ROLE_SCROLL_BAR):
             return self.getZonesFromAccessible(root, rootexts)
 
+        # If this is a status bar, only pursue its children if we cannot
+        # get non-empty text information from the status bar.
+        # See bug #506874 for more details.
+        #
+        if rootrole == pyatspi.ROLE_STATUS_BAR:
+            zones = self.getZonesFromText(root, rootexts)
+            if len(zones):
+                return zones
+
         # Otherwise, dig deeper.
         #
         # We'll include page tabs: while they are parents, their extents do
@@ -1628,6 +1671,7 @@ class Context:
                         regionWithFocus.cursorOffset += \
                             len(zone.words[wordIndex].string.decode("UTF-8"))
                 regionWithFocus.cursorOffset += self.charIndex
+                regionWithFocus.repositionCursor()
                 break
 
         return [regions, regionWithFocus]
