@@ -19,9 +19,9 @@
 
 """Provides a SpeechServer factory for gnome-speech drivers."""
 
-__id__        = "$Id: gnomespeechfactory.py 3985 2008-06-17 19:11:50Z wwalker $"
-__version__   = "$Revision: 3985 $"
-__date__      = "$Date: 2008-06-17 15:11:50 -0400 (Tue, 17 Jun 2008) $"
+__id__        = "$Id: gnomespeechfactory.py 3783 2008-03-28 23:49:14Z wwalker $"
+__version__   = "$Revision: 3783 $"
+__date__      = "$Date: 2008-03-28 19:49:14 -0400 (Fri, 28 Mar 2008) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -52,50 +52,6 @@ from orca_i18n import _           # for gettext support
 ORBit.load_typelib('GNOME_Speech')
 import GNOME.Speech, GNOME__POA.Speech
 
-def _matchLanguage(desired, actual):
-    """Compares a desired language to an actual language and returns
-    a numerical value representing the degree of match.  The expected
-    language format is a string similar to the following, where the
-    delimiter can be '-' '_' or '.':
-
-      de-DE (German for Germany)
-      en-US (English as used in the United States)
-
-    Additional extensions for variants or dialects are allowed as well.
-
-    The return value is one of the following:
-
-      0 - no match at all
-      1 - some match
-      2 - exact match
-    """
-
-    if desired == actual:
-        return 2
-    if not desired or not actual:
-        return 0
-
-    # Break the strings into arrays where the delimiters are
-    # - or _.  Also lowercase everything to help with string 
-    # compares.
-    #
-    desired = desired.lower().replace("-","_").replace(".","_").split("_")
-    actual = actual.lower().replace("-","_").replace(".","_").split("_")
-
-    matchCount = 0
-    for i in range(0, min(len(desired), len(actual))):
-        if desired[i] == actual[i]:
-            matchCount += 1
-        else:
-            break
-
-    if matchCount == len(desired):
-        return 2
-    elif matchCount:
-        return 1
-    else:
-        return 0
-        
 class _SayAll:
     def __init__(self, iterator, context, utteranceId, progressCallback):
         self.utteranceIterator   = iterator
@@ -455,16 +411,7 @@ class SpeechServer(speechserver.SpeechServer):
         voices = self.__driver.getAllVoices()
         foundVoices = []
         for voice in voices:
-            match = 0
-            for language in languages:
-                match = max(match, _matchLanguage(voice.language, language))
-
-            # Exact matches get put at the beginning, inexact get added to
-            # the end.
-            #
-            if match == 2:
-                foundVoices.insert(0, voice)
-            elif match:
+            if voice.language in languages:
                 foundVoices.append(voice)
 
         # If we didn't find any matches, well...punt.
@@ -506,16 +453,12 @@ class SpeechServer(speechserver.SpeechServer):
 
         speaker = _Speaker(s._narrow(GNOME.Speech.Speaker))
 
-        # Turn off punctuation.  We do this because we want to handle
-        # spoken punctuation on our own.  Only do so if "punctuation mode" 
-        # is available (see # bug #528797).
+        # Turn off punctuation if the speaker allows us to do so.  We
+        # do this because we want to handle spoken punctuation on our
+        # own.
         #
         try:
-            parameters = speaker.getSupportedParameters()
-            for parameter in parameters:
-                if parameter.name == "punctuation mode":
-                    speaker.setParameterValue("punctuation mode", 0.0)
-                    break
+            speaker.setParameterValue("punctuation mode", 0.0)
         except:
             pass
 
@@ -595,6 +538,13 @@ class SpeechServer(speechserver.SpeechServer):
                             self.__sayAll.currentContext,
                             speechserver.SayAllContext.COMPLETED)
                         self.__sayAll = None
+                    except:
+                        # This is here to deal with the situation where speech
+                        # might have stopped and __sayAll was set to None.
+                        # If we don't handle those cases, Say All can stop
+                        # working.
+                        #
+                        pass
 
         rerun = True
 
