@@ -20,9 +20,9 @@
 """Exposes a dictionary, keynames, that maps key events
 into localized words."""
 
-__id__        = "$Id: keynames.py 2254 2007-04-05 13:21:02Z wwalker $"
-__version__   = "$Revision: 2254 $"
-__date__      = "$Date: 2007-04-05 09:21:02 -0400 (Thu, 05 Apr 2007) $"
+__id__        = "$Id: keynames.py 3741 2008-03-20 22:07:04Z wwalker $"
+__version__   = "$Revision: 3741 $"
+__date__      = "$Date: 2008-03-20 18:07:04 -0400 (Thu, 20 Mar 2008) $"
 __copyright__ = "Copyright (c) 2006-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -100,6 +100,18 @@ __keynames["SunF36"]       = _("F 11")
 #
 __keynames["SunF37"]       = _("F 12")
 
+# Translators: this is the spoken word for the space character
+#
+__keynames["space"] = _("space")
+
+# Translators: this is the spoken word for the BackSpace key
+#
+__keynames["BackSpace"] = _("backspace")
+
+#translators: this is the spoken word for Return key
+#
+__keynames["Return"] = _("return")
+
 def getKeyName(key):
     """Given a keyboard key, return its name as people might refer to it
     in ordinary conversation.
