@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2007 Sun Microsystems Inc.
+# Copyright 2004-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -19,10 +19,10 @@
 
 """The main module for the Orca screen reader."""
 
-__id__        = "$Id: orca.py 3375 2007-12-14 16:48:23Z joanied $"
-__version__   = "$Revision: 3375 $"
-__date__      = "$Date: 2007-12-14 11:48:23 -0500 (Fri, 14 Dec 2007) $"
-__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
+__id__        = "$Id: orca.py 3819 2008-04-07 19:20:05Z wwalker $"
+__version__   = "$Revision: 3819 $"
+__date__      = "$Date: 2008-04-07 15:20:05 -0400 (Mon, 07 Apr 2008) $"
+__copyright__ = "Copyright (c) 2004-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 # We're going to force the name of the app to "orca" so pygtk
@@ -41,6 +41,7 @@ try:
     # the desktop is not running.
     #
     import gtk
+    import mouse_review
 except:
     pass
 
@@ -54,6 +55,7 @@ import unicodedata
 import pyatspi
 import braille
 import debug
+import dbusserver
 import httpserver
 import keynames
 import keybindings
@@ -141,6 +143,17 @@ def _switchToNextPresentationManager(script=None, inputEvent=None):
     _switchToPresentationManager(_currentPresentationManager + 1)
     return True
 
+def getScriptForApp(app):
+    """Get the script for the given application object from the current
+    presentation manager.
+
+    Arguments:
+    - app: An application accessible.
+
+    Returns a Script instance.
+    """
+    return _PRESENTATION_MANAGERS[_currentPresentationManager].getScript(app)
+
 ########################################################################
 #                                                                      #
 # METHODS TO HANDLE APPLICATION LIST AND FOCUSED OBJECTS               #
@@ -316,7 +329,7 @@ _keyBindings = None
 #
 _orcaModifierPressed = False
 
-def _isPrintableKey(event_string):
+def isPrintableKey(event_string):
     """Return an indication of whether this is an alphanumeric or
        punctuation key.
 
@@ -334,10 +347,10 @@ def _isPrintableKey(event_string):
                 and (unicodeString.isalnum() or unicodeString.isspace()
                 or unicodedata.category(unicodeString)[0] in ('P', 'S'))
     debug.println(debug.LEVEL_FINEST,
-                  "orca._isPrintableKey: returning: %s" % reply)
+                  "orca.isPrintableKey: returning: %s" % reply)
     return reply
 
-def _isModifierKey(event_string):
+def isModifierKey(event_string):
     """Return an indication of whether this is a modifier key.
 
     Arguments:
@@ -352,10 +365,10 @@ def _isModifierKey(event_string):
 
     reply = event_string in modifierKeys
     debug.println(debug.LEVEL_FINEST,
-                  "orca._isModifierKey: returning: %s" % reply)
+                  "orca.isModifierKey: returning: %s" % reply)
     return reply
 
-def _isLockingKey(event_string):
+def isLockingKey(event_string):
     """Return an indication of whether this is a locking key.
 
     Arguments:
@@ -369,10 +382,10 @@ def _isLockingKey(event_string):
     reply = event_string in lockingKeys \
             and not event_string in settings.orcaModifierKeys
     debug.println(debug.LEVEL_FINEST,
-                  "orca._isLockingKey: returning: %s" % reply)
+                  "orca.isLockingKey: returning: %s" % reply)
     return reply
 
-def _isFunctionKey(event_string):
+def isFunctionKey(event_string):
     """Return an indication of whether this is a function key.
 
     Arguments:
@@ -390,10 +403,10 @@ def _isFunctionKey(event_string):
 
     reply = event_string in functionKeys
     debug.println(debug.LEVEL_FINEST,
-                  "orca._isFunctionKey: returning: %s" % reply)
+                  "orca.isFunctionKey: returning: %s" % reply)
     return reply
 
-def _isActionKey(event_string):
+def isActionKey(event_string):
     """Return an indication of whether this is an action key.
 
     Arguments:
@@ -407,10 +420,10 @@ def _isActionKey(event_string):
 
     reply = event_string in actionKeys
     debug.println(debug.LEVEL_FINEST,
-                  "orca._isActionKey: returning: %s" % reply)
+                  "orca.isActionKey: returning: %s" % reply)
     return reply
 
-def _isNavigationKey(event_string):
+def isNavigationKey(event_string):
     """Return an indication of whether this is a navigation (arrow) key
     or if the user has the Orca modifier key held done.
 
@@ -426,7 +439,7 @@ def _isNavigationKey(event_string):
     reply = (event_string in navigationKeys) or _orcaModifierPressed
 
     debug.println(debug.LEVEL_FINEST,
-                  "orca._isNavigationKey: returning: %s" % reply)
+                  "orca.isNavigationKey: returning: %s" % reply)
     return reply
 
 class KeyEventType:
@@ -459,7 +472,7 @@ class KeyEventType:
     def __init__(self):
         pass
 
-def _keyEcho(event):
+def keyEcho(event):
     """If the keyEcho setting is enabled, check to see what type of key
     event it is and echo it via speech, if the user wants that type of
     key echoed.
@@ -480,7 +493,7 @@ def _keyEcho(event):
 
     event_string = event.event_string
     debug.println(debug.LEVEL_FINEST,
-                  "orca._keyEcho: string to echo: %s" % event_string)
+                  "orca.keyEcho: string to echo: %s" % event_string)
 
     # If key echo is enabled, then check to see what type of key event
     # it is and echo it via speech, if the user wants that type of key
@@ -488,22 +501,22 @@ def _keyEcho(event):
     #
     if settings.enableKeyEcho:
 
-        if _isModifierKey(event_string):
+        if isModifierKey(event_string):
             if not settings.enableModifierKeys:
                 return
             eventType = KeyEventType.MODIFIER
 
-        elif _isNavigationKey(event_string):
+        elif isNavigationKey(event_string):
             if not settings.enableNavigationKeys:
                 return
             eventType = KeyEventType.NAVIGATION
 
-        elif _isPrintableKey(event_string):
+        elif isPrintableKey(event_string):
             if not settings.enablePrintableKeys:
                 return
             eventType = KeyEventType.PRINTABLE
 
-        elif _isLockingKey(event_string):
+        elif isLockingKey(event_string):
             if not settings.enableLockingKeys:
                 return
             eventType = KeyEventType.LOCKING
@@ -528,23 +541,23 @@ def _keyEcho(event):
                 #    eventType = KeyEventType.LOCKING_LOCKED
                 pass
 
-        elif _isFunctionKey(event_string):
+        elif isFunctionKey(event_string):
             if not settings.enableFunctionKeys:
                 return
             eventType = KeyEventType.FUNCTION
 
-        elif _isActionKey(event_string):
+        elif isActionKey(event_string):
             if not settings.enableActionKeys:
                 return
             eventType = KeyEventType.ACTION
 
         else:
             debug.println(debug.LEVEL_FINEST,
-                  "orca._keyEcho: event string not handled: %s" % event_string)
+                  "orca.keyEcho: event string not handled: %s" % event_string)
             return
 
         debug.println(debug.LEVEL_FINEST,
-                      "orca._keyEcho: speaking: %s" % event_string)
+                      "orca.keyEcho: speaking: %s" % event_string)
 
         # We keep track of the time as means to let others know that
         # we are probably echoing a key and should not be interrupted.
@@ -553,27 +566,44 @@ def _keyEcho(event):
 
         speech.speakKeyEvent(event_string, eventType)
 
+def _setClickCount(inputEvent):
+    """Sets the count of the number of clicks a user has made to one
+    of the non-modifier keys on the keyboard.
+
+    Arguments:
+    - inputEvent: the current input event.
+    """
+
+    lastInputEvent = orca_state.lastNonModifierKeyEvent
+
+    if inputEvent.type == pyatspi.KEY_RELEASED_EVENT:
+        pass
+    elif not isinstance(inputEvent, KeyboardEvent):
+        orca_state.clickCount = 0
+    elif not isinstance(lastInputEvent, KeyboardEvent):
+        orca_state.clickCount = 1
+    elif (lastInputEvent.hw_code != inputEvent.hw_code) or \
+         (lastInputEvent.modifiers != inputEvent.modifiers):
+        orca_state.clickCount = 1
+    elif (inputEvent.time - lastInputEvent.time) < \
+           settings.doubleClickTimeout:
+        # Cap the possible number of clicks at 3.
+        #
+        if orca_state.clickCount < 3:
+            orca_state.clickCount += 1
+    else:
+        orca_state.clickCount = 1
+
 def _processKeyCaptured(event):
     """Called when a new key event arrives and orca_state.capturingKeys=True.
     (used for key bindings redefinition)
     """
 
     if event.type == 0:
-        if _isModifierKey(event.event_string) \
-           or _isLockingKey(event.event_string):
+        if isModifierKey(event.event_string) \
+           or isLockingKey(event.event_string):
             return True
         else:
-            # We want to capture this event, after first doing a little
-            # clean up.  Eliminate modifiers we're not interested in.
-            #
-            mask = (1 << settings.MODIFIER_ORCA |
-                    1 << pyatspi.MODIFIER_ALT |
-                    1 << pyatspi.MODIFIER_SHIFT |
-                    1 << pyatspi.MODIFIER_CONTROL |
-                    1 << pyatspi.MODIFIER_META2 |
-                    1 << pyatspi.MODIFIER_META3)
-            event.modifiers = event.modifiers & mask
-
             # We want the keyname rather than the printable character.
             # If it's not on the keypad, get the name of the unshifted
             # character. (i.e. "1" instead of "!")
@@ -581,7 +611,8 @@ def _processKeyCaptured(event):
             keymap = gtk.gdk.keymap_get_default()
             entries = keymap.get_entries_for_keycode(event.hw_code)
             event.event_string = gtk.gdk.keyval_name(entries[0][0])
-            if event.event_string.startswith("KP"):
+            if event.event_string.startswith("KP") and \
+               event.event_string != "KP_Enter":
                 name = gtk.gdk.keyval_name(entries[1][0])
                 if name.startswith("KP"):
                     event.event_string = name
@@ -667,8 +698,9 @@ def _processKeyboardEvent(event):
 
         # If learn mode is enabled, it will echo the keys.
         #
-        if not settings.learnModeEnabled:
-            _keyEcho(keyboardEvent)
+        if not settings.learnModeEnabled and \
+           orca_state.activeScript.echoKey(keyboardEvent):
+            keyEcho(keyboardEvent)
 
     elif isOrcaModifier \
         and (keyboardEvent.type == pyatspi.KEY_RELEASED_EVENT):
@@ -678,6 +710,19 @@ def _processKeyboardEvent(event):
         keyboardEvent.modifiers = keyboardEvent.modifiers \
                                   | (1 << settings.MODIFIER_ORCA)
 
+    orca_state.lastInputEvent = keyboardEvent
+
+    # If this is a key event for a non-modifier key, save a handle to it.
+    # This is needed to help determine user actions when a multi-key chord
+    # has been pressed, and we might get the key events in different orders.
+    # See comment #15 of bug #435201 for more details.  We also want to
+    # store the "click count" for the purpose of supporting keybindings
+    # with unique behaviors when double- or triple-clicked.
+    #
+    if not isModifierKey(keyboardEvent.event_string):
+        _setClickCount(keyboardEvent)
+        orca_state.lastNonModifierKeyEvent = keyboardEvent
+
     # Orca gets first stab at the event.  Then, the presenter gets
     # a shot. [[[TODO: WDW - might want to let the presenter try first?
     # The main reason this is staying as is is that we may not want
@@ -693,12 +738,10 @@ def _processKeyboardEvent(event):
                 consumed = _PRESENTATION_MANAGERS[_currentPresentationManager].\
                            processKeyboardEvent(keyboardEvent)
             if (not consumed) and settings.learnModeEnabled:
-                if keyboardEvent.type \
-                    == pyatspi.KEY_PRESSED_EVENT:
-                    clickCount = orca_state.activeScript.getClickCount(\
-                                                orca_state.lastInputEvent,
-                                                keyboardEvent)
-                    if clickCount == 2:
+                if keyboardEvent.type == pyatspi.KEY_PRESSED_EVENT:
+                    clickCount = orca_state.activeScript.getClickCount()
+                    if isPrintableKey(keyboardEvent.event_string) \
+                       and clickCount == 2:
                         orca_state.activeScript.phoneticSpellCurrentItem(\
                             keyboardEvent.event_string)
                     else:
@@ -713,16 +756,6 @@ def _processKeyboardEvent(event):
     except:
         debug.printException(debug.LEVEL_SEVERE)
 
-    orca_state.lastInputEvent = keyboardEvent
-
-    # If this is a key event for a non-modifier key, save a handle to it.
-    # This is needed to help determine user actions when a multi-key chord
-    # has been pressed, and we might get the key events in different orders.
-    # See comment #15 of bug #435201 for more details.
-    #
-    if not _isModifierKey(keyboardEvent.event_string):
-        orca_state.lastNonModifierKeyEvent = keyboardEvent
-
     return consumed or isOrcaModifier
 
 ########################################################################
@@ -797,6 +830,7 @@ def loadUserSettings(script=None, inputEvent=None):
 
     # Shutdown the output drivers and give them a chance to die.
     #
+    dbusserver.shutdown()
     httpserver.shutdown()
     speech.shutdown()
     braille.shutdown()
@@ -870,6 +904,14 @@ def loadUserSettings(script=None, inputEvent=None):
         debug.println(debug.LEVEL_CONFIGURATION,
                       "Magnification module has NOT been initialized.")
 
+
+    # I'm not sure where else this should go. But it doesn't really look
+    # right here.
+    try:
+        mouse_review.mouse_reviewer.toggle(on=settings.enableMouseReview)
+    except NameError:
+        pass
+
     # We don't want the Caps_Lock modifier to act as a locking
     # modifier if it used as the Orca modifier key.  In addition, if
     # the KP_Insert key is used as the Orca modifier key, we want to
@@ -900,6 +942,7 @@ def loadUserSettings(script=None, inputEvent=None):
 
     showMainWindowGUI()
 
+    dbusserver.init()
     httpserver.init()
 
     return True
@@ -973,7 +1016,7 @@ def _showPreferencesConsole(script=None, inputEvent=None):
                             globals(),
                             locals(),
                             [''])
-        module.showPreferencesUI()
+        module.showPreferencesUI(_commandLineSettings)
     except:
         debug.printException(debug.LEVEL_SEVERE)
 
