@@ -19,9 +19,9 @@
 
 """Custom script for Evolution."""
 
-__id__        = "$Id: Evolution.py 3354 2007-12-08 02:40:44Z eitani $"
-__version__   = "$Revision: 3354 $"
-__date__      = "$Date: 2007-12-07 21:40:44 -0500 (Fri, 07 Dec 2007) $"
+__id__        = "$Id: Evolution.py 3784 2008-04-01 01:24:54Z richb $"
+__version__   = "$Revision: 3784 $"
+__date__      = "$Date: 2008-03-31 21:24:54 -0400 (Mon, 31 Mar 2008) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -36,6 +36,7 @@ import orca.braille as braille
 import orca.orca as orca
 import orca.orca_state as orca_state
 import orca.speech as speech
+import orca.speechgenerator as speechgenerator
 import orca.speechserver as speechserver
 import orca.settings as settings
 import orca.chnames as chnames
@@ -65,7 +66,11 @@ class WhereAmI(where_am_I.WhereAmI):
 
         if action:
             for i in range(0, action.nActions):
-                if action.getName(i) == "toggle":
+                # Translators: this is the action name for
+                # the 'toggle' action. It must be the same
+                # string used in the *.po file for gail.
+                #
+                if action.getName(i) in ["toggle", _("toggle")]:
                     if not obj.getState().contains(pyatspi.STATE_CHECKED):
                         notChecked = True
                     break
@@ -232,6 +237,45 @@ class WhereAmI(where_am_I.WhereAmI):
 
         return [textContents, startOffset, endOffset]
 
+class SpeechGenerator(speechgenerator.SpeechGenerator):
+    """Overrides _getSpeechForTableCell so that, if this is an expanded 
+       table cell,  we can strip off the "0 items".
+    """
+
+    def __init__(self, script):
+        speechgenerator.SpeechGenerator.__init__(self, script)
+
+    def _getSpeechForTableCell(self, obj, already_focused):
+        """Get the speech utterances for a single table cell
+
+        Arguments:
+        - obj: the table
+        - already_focused: False if object just received focus
+
+        Returns a list of utterances to be spoken for the object.
+        """
+
+        utterances = speechgenerator.SpeechGenerator.\
+                      _getSpeechForTableCell(self, obj, already_focused)
+
+        # Check that we are in a table cell in the mail message header list.
+        # If we are and this table cell has an expanded state, and the first
+        # token of the last utterances is "0", then strip off that last 
+        # utterance ("0 items"). See bug #432308 for more details.
+        #
+        rolesList = [pyatspi.ROLE_TABLE_CELL, \
+                     pyatspi.ROLE_TREE_TABLE, \
+                     pyatspi.ROLE_UNKNOWN]
+        if self._script.isDesiredFocusedItem(obj, rolesList):
+            state = obj.getState()
+            if state and state.contains(pyatspi.STATE_EXPANDABLE):
+                if state.contains(pyatspi.STATE_EXPANDED):
+                    tokens = utterances[-1].split()
+                    if tokens[0] == "0":
+                        utterances = utterances[0:-1]
+
+        return utterances
+
 ########################################################################
 #                                                                      #
 # The Evolution script class.                                          #
@@ -271,6 +315,11 @@ class Script(default.Script):
         self.lastMessageColumn = -1
         self.lastMessageRow = -1
 
+        # The last locusOfFocusChanged roles hierarchy.
+        #
+        self.lastRolesList = []
+        self.rolesList = []
+
         # By default, don't present if Evolution is not the active application.
         #
         self.presentIfInactive = False
@@ -314,6 +363,12 @@ class Script(default.Script):
             #
             _("calendar event"))
 
+    def getSpeechGenerator(self):
+        """Returns the speech generator for this script.
+        """
+
+        return SpeechGenerator(self)
+
     def getWhereAmI(self):
         """Returns the "where am I" class for this script.
         """
@@ -655,7 +710,7 @@ class Script(default.Script):
 
         if callbackType == speechserver.SayAllContext.PROGRESS:
             #print "PROGRESS", context.utterance, context.currentOffset
-            pass
+            return
         elif callbackType == speechserver.SayAllContext.INTERRUPTED:
             #print "INTERRUPTED", context.utterance, context.currentOffset
             offset = context.currentOffset
@@ -673,6 +728,46 @@ class Script(default.Script):
             obj.queryText().setCaretOffset(context.currentOffset)
             orca.setLocusOfFocus(None, obj, False)
 
+        # If there is a selection, clear it. See bug #489504 for more details.
+        # This is not straight forward with Evolution. all the text is in
+        # an HTML panel which contains multiple panels, each containing a
+        # single text object.
+        #
+        panel = obj.parent
+        htmlPanel = panel.parent
+        for i in range(0, htmlPanel.childCount):
+            panel = htmlPanel.getChildAtIndex(i)
+            if panel != None:
+                textObj = panel.getChildAtIndex(0)
+                try:
+                    text = textObj.queryText()
+                except:
+                    pass
+                else:
+                    if text.getNSelections():
+                        text.removeSelection(0)
+
+    def presentMessageLine(self, obj, newLocusOfFocus):
+        """Speak/braille the line at the current text caret offset.
+        """
+
+        [string, caretOffset, startOffset] = self.getTextLineAtCaret(obj)
+        self.updateBraille(newLocusOfFocus)
+        if settings.enableSpeechIndentation:
+            self.speakTextIndentation(obj, string)
+        line = self.adjustForRepeats(string)
+
+        if self.speakNewLine(obj):
+            speech.speak(chnames.getCharacterName("\n"), None, False)
+
+        if self.speakBlankLine(obj):
+            # Translators: "blank" is a short word to mean the
+            # user has navigated to an empty line.
+            #
+            speech.speak(_("blank"), None, False)
+        else:
+            speech.speak(line, None, False)
+
     def sayAll(self, inputEvent):
         """Speak all the text associated with the text object that has
            focus. We have to define our own method here because Evolution
@@ -728,6 +823,11 @@ class Script(default.Script):
         brailleGen = self.brailleGenerator
         speechGen = self.speechGenerator
 
+        # Save the previous role hierarchy list for possible comparison 
+        # in section 8).
+        #
+        self.lastRolesList = self.rolesList
+
         debug.printObjectEvent(self.debugLevel,
                                event,
                                debug.getAccessibleDetails(event.source))
@@ -747,33 +847,20 @@ class Script(default.Script):
         # it's a line in the mail message and we get the utterances to
         # speak for that Text.
 
-        rolesList = [pyatspi.ROLE_TEXT, \
-                     pyatspi.ROLE_PANEL, \
-                     pyatspi.ROLE_UNKNOWN]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        self.rolesList = [pyatspi.ROLE_TEXT, \
+                          pyatspi.ROLE_PANEL, \
+                          pyatspi.ROLE_UNKNOWN, \
+                          pyatspi.ROLE_PANEL, \
+                          pyatspi.ROLE_SCROLL_PANE, \
+                          pyatspi.ROLE_FILLER, \
+                          pyatspi.ROLE_SPLIT_PANE]
+        if self.isDesiredFocusedItem(event.source, self.rolesList):
             debug.println(self.debugLevel,
                           "evolution.locusOfFocusChanged - mail view: " \
                           + "current message pane: " \
                           + "individual lines of text.")
 
-            [string, caretOffset, startOffset] = \
-                self.getTextLineAtCaret(event.source)
-            self.updateBraille(newLocusOfFocus)
-            if settings.enableSpeechIndentation:
-                self.speakTextIndentation(event.source, string)
-            line = self.adjustForRepeats(string)
-
-            if self.speakNewLine(event.source):
-                speech.speak(chnames.getCharacterName("\n"), None, False)
-
-            if self.speakBlankLine(event.source):
-                # Translators: "blank" is a short word to mean the
-                # user has navigated to an empty line.
-                #
-                speech.speak(_("blank"), None, False)
-            else:
-                speech.speak(line, None, False)
-
+            self.presentMessageLine(event.source, newLocusOfFocus)
             return
 
 
@@ -797,11 +884,11 @@ class Script(default.Script):
         # component and that the final component (with no children) is of
         # role TEXT.
 
-        rolesList = [pyatspi.ROLE_TEXT, \
-                     pyatspi.ROLE_PANEL, \
-                     pyatspi.ROLE_TABLE_CELL]
+        self.rolesList = [pyatspi.ROLE_TEXT, \
+                          pyatspi.ROLE_PANEL, \
+                          pyatspi.ROLE_TABLE_CELL]
         if settings.readTableCellRow \
-            and (self.isDesiredFocusedItem(event.source, rolesList)):
+            and (self.isDesiredFocusedItem(event.source, self.rolesList)):
             debug.println(self.debugLevel,
                           "evolution.locusOfFocusChanged - mail view: " \
                           + "current message pane: " \
@@ -849,10 +936,10 @@ class Script(default.Script):
         # the message list and the order in which they appear, so that
         # Orca will just speak the ones that they are interested in.
 
-        rolesList = [pyatspi.ROLE_TABLE_CELL, \
-                     pyatspi.ROLE_TREE_TABLE]
+        self.rolesList = [pyatspi.ROLE_TABLE_CELL, \
+                          pyatspi.ROLE_TREE_TABLE]
         if settings.readTableCellRow \
-            and (self.isDesiredFocusedItem(event.source, rolesList)):
+            and (self.isDesiredFocusedItem(event.source, self.rolesList)):
             debug.println(self.debugLevel,
                           "evolution.locusOfFocusChanged - mail view: " \
                           + "message header list.")
@@ -871,6 +958,16 @@ class Script(default.Script):
             column = \
                 parentTable.getColumnAtIndex(event.source.getIndexInParent())
 
+            # If we are on the same row, then just speak/braille the table
+            # cell as if settings.readTableCellRow was False. 
+            # See bug #503874 for more details.
+            #
+            if self.lastMessageRow == row:
+                default.Script.locusOfFocusChanged(self, event,
+                                           oldLocusOfFocus, newLocusOfFocus)
+                settings.readTableCellRow = True
+                return
+
             # This is an indication of whether we should speak all the table
             # cells (the user has moved focus up or down the list, or just
             # deleted a message), or just the current one (focus has moved
@@ -934,6 +1031,12 @@ class Script(default.Script):
 
                     checkbox = False
                     toRead = True
+                    # Whether or not we want to replace the cell's column
+                    # header with a more user-friendly alternative name.
+                    # Currently we only do this with the "status" column,
+                    # which we replace with "unread" if it is not checked.
+                    #
+                    useAlternativeName = False
                     try:
                         action = cell.queryAction()
                     except NotImplementedError:
@@ -941,7 +1044,11 @@ class Script(default.Script):
 
                     if action:
                         for j in range(0, action.nActions):
-                            if action.getName(j) == "toggle":
+                            # Translators: this is the action name for
+                            # the 'toggle' action. It must be the same
+                            # string used in the *.po file for gail.
+                            #
+                            if action.getName(j) in ["toggle", _("toggle")]:
                                 checkbox = True
                                 checked = cell.getState().contains( \
                                     pyatspi.STATE_CHECKED)
@@ -953,6 +1060,15 @@ class Script(default.Script):
                                     #
                                     if header.name == _("Status"):
                                         toRead = not checked
+                                        useAlternativeName = True
+                                        break
+                                    # Translators: this is the name of the
+                                    # flagged column header in the message
+                                    # list in Evolution.  The name needs to
+                                    # match what Evolution is using.
+                                    #
+                                    elif header.name == _("Flagged"):
+                                        toRead = checked
                                         break
                                     if not checked:
                                         toRead = False
@@ -962,7 +1078,8 @@ class Script(default.Script):
                         # Speak/braille the column header for this table cell
                         # if it has focus (unless it's a checkbox).
                         #
-                        if not checkbox and verbose:
+                        if (verbose or (checkbox and column == i)) \
+                           and not useAlternativeName:
                             settings.brailleVerbosityLevel = \
                                 settings.VERBOSITY_LEVEL_BRIEF
                             settings.speechVerbosityLevel = \
@@ -1020,8 +1137,20 @@ class Script(default.Script):
                         elif header.name == _("Attachment"):
                             text = header.name
                             utterances = [ text ]
-                            brailleRegions.append(braille.Region(text))
-                            brailleRegions.append(braille.Region(" "))
+                            if column != i:
+                                brailleRegions.append(braille.Region(text))
+                                brailleRegions.append(braille.Region(" "))
+                        # Translators: this is the name of the
+                        # flagged column header in the message
+                        # list in Evolution.  The name needs to
+                        # match what Evolution is using.
+                        #
+                        elif header.name == _("Flagged"):
+                            text = header.name
+                            utterances = [ text ]
+                            if column != i:
+                                brailleRegions.append(braille.Region(text))
+                                brailleRegions.append(braille.Region(" "))
                         else:
                             brailleRegions.extend(cellRegions)
                             brailleRegions.append(braille.Region(" "))
@@ -1065,9 +1194,9 @@ class Script(default.Script):
         # which is determined by the number of children in the parent Calendar
         # View's table.
 
-        rolesList = [rolenames.ROLE_CALENDAR_EVENT, \
-                     rolenames.ROLE_CALENDAR_VIEW]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        self.rolesList = [rolenames.ROLE_CALENDAR_EVENT, \
+                          rolenames.ROLE_CALENDAR_VIEW]
+        if self.isDesiredFocusedItem(event.source, self.rolesList):
             debug.println(self.debugLevel,
                           "evolution.locusOfFocusChanged - calendar view: " \
                           + "day view: tabbing to day with appts.")
@@ -1130,10 +1259,10 @@ class Script(default.Script):
         # which is determined by the number of children in the parent Calendar
         # View's table.
 
-        rolesList = [pyatspi.ROLE_UNKNOWN, \
-                     pyatspi.ROLE_TABLE, \
-                     rolenames.ROLE_CALENDAR_VIEW]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        self.rolesList = [pyatspi.ROLE_UNKNOWN, \
+                          pyatspi.ROLE_TABLE, \
+                          rolenames.ROLE_CALENDAR_VIEW]
+        if self.isDesiredFocusedItem(event.source, self.rolesList):
             debug.println(self.debugLevel,
                       "evolution.locusOfFocusChanged - calendar view: " \
                       + "day view: moving with arrow keys.")
@@ -1208,11 +1337,11 @@ class Script(default.Script):
         # NOTE: assumes there is only one "page tab list" in the "filler"
         # component.
 
-        rolesList = [pyatspi.ROLE_TABLE_CELL, \
-                     pyatspi.ROLE_TABLE, \
-                     pyatspi.ROLE_UNKNOWN, \
-                     pyatspi.ROLE_SCROLL_PANE]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        self.rolesList = [pyatspi.ROLE_TABLE_CELL, \
+                          pyatspi.ROLE_TABLE, \
+                          pyatspi.ROLE_UNKNOWN, \
+                          pyatspi.ROLE_SCROLL_PANE]
+        if self.isDesiredFocusedItem(event.source, self.rolesList):
             debug.println(self.debugLevel,
                       "evolution.locusOfFocusChanged - preferences dialog: " \
                       + "table cell in options list.")
@@ -1236,17 +1365,17 @@ class Script(default.Script):
         # rather than just speak/braille "button", output something a
         # little more useful.
 
-        rolesList = [pyatspi.ROLE_PUSH_BUTTON, \
-                     pyatspi.ROLE_PANEL, \
-                     pyatspi.ROLE_FILLER, \
-                     pyatspi.ROLE_FILLER, \
-                     pyatspi.ROLE_SPLIT_PANE, \
-                     pyatspi.ROLE_FILLER, \
-                     pyatspi.ROLE_FILLER, \
-                     pyatspi.ROLE_FILLER, \
-                     pyatspi.ROLE_FILLER, \
-                     pyatspi.ROLE_DIALOG]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        self.rolesList = [pyatspi.ROLE_PUSH_BUTTON, \
+                          pyatspi.ROLE_PANEL, \
+                          pyatspi.ROLE_FILLER, \
+                          pyatspi.ROLE_FILLER, \
+                          pyatspi.ROLE_SPLIT_PANE, \
+                          pyatspi.ROLE_FILLER, \
+                          pyatspi.ROLE_FILLER, \
+                          pyatspi.ROLE_FILLER, \
+                          pyatspi.ROLE_FILLER, \
+                          pyatspi.ROLE_DIALOG]
+        if self.isDesiredFocusedItem(event.source, self.rolesList):
             debug.println(self.debugLevel,
                           "evolution.locusOfFocusChanged - mail insert " \
                           + "attachment dialog: unlabelled button.")
@@ -1274,27 +1403,39 @@ class Script(default.Script):
         # Note that this drops through to then use the default event
         # processing in the parent class for this "focus:" event.
 
-        rolesList = [pyatspi.ROLE_TEXT, \
-                     pyatspi.ROLE_PANEL, \
-                     pyatspi.ROLE_PANEL, \
-                     pyatspi.ROLE_SCROLL_PANE]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        self.rolesList = [pyatspi.ROLE_TEXT, \
+                          pyatspi.ROLE_PANEL, \
+                          pyatspi.ROLE_UNKNOWN, \
+                          pyatspi.ROLE_PANEL, \
+                          pyatspi.ROLE_SCROLL_PANE, \
+                          pyatspi.ROLE_FILLER, \
+                          pyatspi.ROLE_PANEL]
+        if self.isDesiredFocusedItem(event.source, self.rolesList):
             debug.println(self.debugLevel,
                           "evolution.locusOfFocusChanged - mail " \
                           + "compose window: message area.")
 
+            # We are getting extra (bogus?) "focus:" events from Evolution
+            # when we type the first character at the beginning of a line.
+            # If the last input event was a keyboard event and the parent
+            # of the locusOfFocus and the event.source are the same, and 
+            # the last roles hierarchy is the same as this one and
+            # the last key pressed wasn't a navigation key, then just 
+            # ignore it. See bug #490317 for more details.
+            #
+            if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
+                if self.isSameObject(event.source.parent,
+                                     orca_state.locusOfFocus.parent):
+                    lastKey = orca_state.lastNonModifierKeyEvent.event_string
+                    if self.lastRolesList == self.rolesList and \
+                       lastKey not in ["Left", "Right", "Up", "Down", \
+                                       "Home", "End", "Return"]:
+                        return
+
             self.message_panel = event.source.parent.parent
 
-            if self.speakNewLine(event.source):
-                speech.speak(chnames.getCharacterName("\n"), None, False)
-
-
-            if self.speakBlankLine(event.source):
-                # Translators: "blank" is a short word to mean the
-                # user has navigated to an empty line.
-                #
-                speech.speak(_("blank"), None, False)
-
+            self.presentMessageLine(event.source, newLocusOfFocus)
+            return
 
         # 9) Spell Checking Dialog
         #
@@ -1306,14 +1447,14 @@ class Script(default.Script):
         # surrounding text, to give the user context for the current misspelt
         # word.
 
-        rolesList = [pyatspi.ROLE_TABLE, \
-                    pyatspi.ROLE_SCROLL_PANE, \
-                    pyatspi.ROLE_PANEL, \
-                    pyatspi.ROLE_PANEL, \
-                    pyatspi.ROLE_PANEL, \
-                    pyatspi.ROLE_FILLER, \
-                    pyatspi.ROLE_DIALOG]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        self.rolesList = [pyatspi.ROLE_TABLE, \
+                         pyatspi.ROLE_SCROLL_PANE, \
+                         pyatspi.ROLE_PANEL, \
+                         pyatspi.ROLE_PANEL, \
+                         pyatspi.ROLE_PANEL, \
+                         pyatspi.ROLE_FILLER, \
+                         pyatspi.ROLE_DIALOG]
+        if self.isDesiredFocusedItem(event.source, self.rolesList):
             debug.println(self.debugLevel,
                       "evolution.locusOfFocusChanged - spell checking dialog.")
 
@@ -1372,14 +1513,14 @@ class Script(default.Script):
         # NOTE: it is assumed that the last table cell in the table
         # contains this information.
 
-        rolesList = [pyatspi.ROLE_PUSH_BUTTON, \
-                    pyatspi.ROLE_FILLER, \
-                    pyatspi.ROLE_PANEL, \
-                    pyatspi.ROLE_PANEL, \
-                    pyatspi.ROLE_TABLE_CELL, \
-                    pyatspi.ROLE_TABLE, \
-                    pyatspi.ROLE_PANEL]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        self.rolesList = [pyatspi.ROLE_PUSH_BUTTON, \
+                         pyatspi.ROLE_FILLER, \
+                         pyatspi.ROLE_PANEL, \
+                         pyatspi.ROLE_PANEL, \
+                         pyatspi.ROLE_TABLE_CELL, \
+                         pyatspi.ROLE_TABLE, \
+                         pyatspi.ROLE_PANEL]
+        if self.isDesiredFocusedItem(event.source, self.rolesList):
             debug.println(self.debugLevel,
                           "evolution.locusOfFocusChanged - " \
                           + "mail message area attachments.")
