@@ -23,14 +23,12 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py 3176 2007-10-24 22:40:14Z eitani $"
-__version__   = "$Revision: 3176 $"
-__date__      = "$Date: 2007-10-24 18:40:14 -0400 (Wed, 24 Oct 2007) $"
+__id__        = "$Id: speechgenerator.py 3784 2008-04-01 01:24:54Z richb $"
+__version__   = "$Revision: 3784 $"
+__date__      = "$Date: 2008-03-31 21:24:54 -0400 (Mon, 31 Mar 2008) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import math
-
 import pyatspi
 import debug
 import orca_state
@@ -163,21 +161,12 @@ class SpeechGenerator:
         Returns a list of utterances to be spoken.
         """
 
-        result = self._script.getAcceleratorAndShortcut(obj)
-
-        accelerator = result[0]
-        #shortcut = result[1]
-
         utterances = []
 
-        # [[[TODO: WDW - various stuff preserved while we work out the
-        # desired verbosity here.]]]
-        #
-        #if len(shortcut) > 0:
-        #    utterances.append(_("shortcut") + " " + shortcut)
-        #    utterances.append(accelerator)
+        result = self._script.getKeyBinding(obj)
+        accelerator = result[2]
+
         if len(accelerator) > 0:
-            #utterances += (_("accelerator") + " " + accelerator)
             utterances.append(accelerator)
 
         return utterances
@@ -222,6 +211,34 @@ class SpeechGenerator:
         else:
             return []
 
+    def _getSpeechForAllTextSelection(self, obj):
+        """Check if this object has text associated with it and it's 
+        completely selected.
+
+        Arguments:
+        - obj: the object being presented
+        """
+
+        utterance = []
+        try:
+            textObj = obj.queryText()
+        except:
+            pass
+        else:
+            noOfSelections = textObj.getNSelections()
+            if noOfSelections == 1:
+                [string, startOffset, endOffset] = \
+                   textObj.getTextAtOffset(0, pyatspi.TEXT_BOUNDARY_LINE_START)
+                if startOffset == 0 and endOffset == len(string):
+                    # Translators: when the user selects (highlights) text in
+                    # a document, Orca lets them know this.
+                    #
+                    # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+                    #
+                    utterance = [Q_("text|selected")]
+
+        return utterance
+
     def _debugGenerator(self, generatorName, obj, already_focused, utterances):
         """Prints debug.LEVEL_FINER information regarding the speech generator.
 
@@ -273,6 +290,7 @@ class SpeechGenerator:
             name = self._getSpeechForObjectName(obj)
             if name != label:
                 utterances.extend(name)
+            utterances.extend(self._getSpeechForAllTextSelection(obj))
             utterances.extend(self._getSpeechForObjectRole(obj, role))
 
         utterances.extend(self._getSpeechForObjectAvailability(obj))
@@ -306,7 +324,8 @@ class SpeechGenerator:
         #
         labels = self._script.findUnrelatedLabels(obj)
         for label in labels:
-            utterances.append(label.name)
+            name = self._getSpeechForObjectName(label)
+            utterances.extend(name)
 
         self._debugGenerator("_getSpeechForAlert",
                              obj,
@@ -375,7 +394,11 @@ class SpeechGenerator:
 
         utterances = []
         state = obj.getState()
-        if state.contains(pyatspi.STATE_CHECKED):
+        if state.contains(pyatspi.STATE_INDETERMINATE):
+            # Translators: this represents the state of a checkbox.
+            #
+            checkedState = _("partially checked")
+        elif state.contains(pyatspi.STATE_CHECKED):
             # Translators: this represents the state of a checkbox.
             #
             checkedState = _("checked")
@@ -481,6 +504,10 @@ class SpeechGenerator:
 
         utterances.extend(self._getSpeechForObjectAvailability(obj))
 
+        for child in obj:
+            if child.getRole() == pyatspi.ROLE_TEXT:
+                utterances.extend(self._getSpeechForAllTextSelection(child))
+
         self._debugGenerator("_getSpeechForComboBox",
                              obj,
                              already_focused,
@@ -888,6 +915,8 @@ class SpeechGenerator:
         [text, caretOffset, startOffset] = self._script.getTextLineAtCaret(obj)
         utterances.append(text)
 
+        utterances.extend(self._getSpeechForAllTextSelection(obj))
+
         self._debugGenerator("_getSpeechForText",
                              obj,
                              already_focused,
@@ -1139,40 +1168,7 @@ class SpeechGenerator:
         Returns a list of utterances to be spoken for the object.
         """
 
-        value = obj.queryValue()
-
-        # OK, this craziness is all about trying to figure out the most
-        # meaningful formatting string for the floating point values.
-        # The number of places to the right of the decimal point should
-        # be set by the minimumIncrement, but the minimumIncrement isn't
-        # always set.  So...we'll default the minimumIncrement to 1/100
-        # of the range.  But, if max == min, then we'll just go for showing
-        # them off to two meaningful digits.
-        #
-        try:
-            minimumIncrement = value.minimumIncrement
-        except:
-            minimumIncrement = 0.0
-
-        if minimumIncrement == 0.0:
-            minimumIncrement = (value.maximumValue - value.minimumValue) \
-                               / 100.0
-
-        try:
-            decimalPlaces = max(0, -math.log10(minimumIncrement))
-        except:
-            try:
-                decimalPlaces = max(0, -math.log10(value.minimumValue))
-            except:
-                try:
-                    decimalPlaces = max(0, -math.log10(value.maximumValue))
-                except:
-                    decimalPlaces = 0
-
-        formatter = "%%.%df" % decimalPlaces
-        valueString = formatter % value.currentValue
-        #minString   = formatter % value.minimumValue
-        #maxString   = formatter % value.maximumValue
+        valueString = self._script.getTextForValue(obj)
 
         if already_focused:
             utterances = [valueString]
@@ -1235,7 +1231,18 @@ class SpeechGenerator:
         Returns a list of utterances to be spoken for the object.
         """
 
-        utterances = self._getDefaultSpeech(obj, already_focused)
+        valueString = self._script.getTextForValue(obj)
+
+        if already_focused:
+            utterances = [valueString]
+        else:
+            utterances = []
+            utterances.extend(self._getSpeechForObjectLabel(obj))
+            if not utterances:
+                utterances.extend(self._getSpeechForObjectName(obj))
+            utterances.extend(self._getSpeechForObjectRole(obj))
+            utterances.append(valueString)
+            utterances.extend(self._getSpeechForObjectAvailability(obj))
 
         self._debugGenerator("_getSpeechForSplitPane",
                              obj,
@@ -1300,7 +1307,11 @@ class SpeechGenerator:
                     continue
                 else:
                     for j in range(0, action.nActions):
-                        if action.getName(j) == "toggle":
+                        # Translators: this is the action name for
+                        # the 'toggle' action. It must be the same
+                        # string used in the *.po file for gail.
+                        #
+                        if action.getName(j) in ["toggle", _("toggle")]:
                             hasToggle[i] = True
                             break
 
@@ -1335,14 +1346,20 @@ class SpeechGenerator:
                 debug.println(debug.LEVEL_FINEST,
                     "speechgenerator.__getTableCellUtterances " \
                     + "looking at action %d" % i)
-                if action.getName(i) == "toggle":
+
+                # Translators: this is the action name for
+                # the 'toggle' action. It must be the same
+                # string used in the *.po file for gail.
+                #
+                if action.getName(i) in ["toggle", _("toggle")]:
                     utterances = self._getSpeechForCheckBox(obj,
                                                             already_focused)
                     break
 
         displayedText = self._script.getDisplayedText( \
                           self._script.getRealActiveDescendant(obj))
-        utterances.append(displayedText)
+        if not already_focused and not displayedText in utterances:
+            utterances.append(displayedText)
 
         # If there is no displayed text, check to see if this table cell 
         # contains an icon (image). If yes:
@@ -1358,9 +1375,10 @@ class SpeechGenerator:
             image = None
 
         if (not displayedText or len(displayedText) == 0) and image:
-            if image.imageDescription:
-                utterances.append(image.imageDescription)
-            utterances.extend(self._getSpeechForImage(obj, already_focused))
+            if not already_focused:
+                if image.imageDescription:
+                    utterances.append(image.imageDescription)
+                utterances.extend(self._getSpeechForImage(obj, already_focused))
 
         state = obj.getState()
         if state.contains(pyatspi.STATE_EXPANDABLE):
@@ -1437,6 +1455,12 @@ class SpeechGenerator:
                 if speakAll:
                     for i in range(0, parent_table.nColumns):
                         cell = parent_table.getAccessibleAt(row, i)
+                        if not cell:
+                            debug.println(debug.LEVEL_WARNING,
+                                 "ERROR: speechgenerator." \
+                                 + "_getSpeechForTableCellRow" \
+                                 + " no accessible at (%d, %d)" % (row, i))
+                            continue
                         state = cell.getState()
                         showing = state.contains(pyatspi.STATE_SHOWING)
                         if showing:
@@ -1453,7 +1477,12 @@ class SpeechGenerator:
                                 action = None
                             if action and (label == None or len(label) == 0):
                                 for j in range(0, action.nActions):
-                                    if action.getName(j) == "toggle":
+                                    # Translators: this is the action name for
+                                    # the 'toggle' action. It must be the same
+                                    # string used in the *.po file for gail.
+                                    #
+                                    if action.getName(j) in ["toggle", \
+                                                             _("toggle")]:
                                         accHeader = \
                                             parent_table.getColumnHeader(i)
                                         utterances.append(accHeader.name)
